import yaml,sys,os,glob
import xarray as xr
import pandas as pd
from analogs import analogs_seas_hcst,analogs_seas_fcst
from datetime import datetime

sys.path.insert(1, '../')

from tools import fix_coords, cut_domain, scale, rms_error 

def load_dirs(arch_conf):

    with open(arch_conf) as f:    
        conf = yaml.load(f, Loader=yaml.FullLoader)
        dirs = conf['dirs']
    return(dirs)

def load_fxconf(run_conf):

    with open(run_conf) as f:    
        conf = yaml.load(f, Loader=yaml.FullLoader)
        conf = conf['fixed_params']

    return(conf)


def load_lpconf(run_conf):

    with open(run_conf) as f:    
        conf = yaml.load(f, Loader=yaml.FullLoader)
        conf = conf['loop_params']
        
    return(conf)


def apply_analogs(exp,srcobs,targetobs,run_conf,arch_conf,target_var,region):
    
    outdir,outfile = get_outfile(exp,run_conf)

    if not os.path.isfile(outdir+outfile):
        exp = exp.interp_like(srcobs[target_var].isel(time=0, drop=True), method="nearest")
        print('cutting domain and loading exp data in memory...')    
        exp = cut_domain(exp,get_domain(region,arch_conf)).load()
        print('exp load completed...')    

        print('Applying analogs method')
        if exp.attrs['system-type'] == "hcst":

            downs = analogs_seas_hcst(exp[target_var],
                         srcobs[target_var],
                         targetobs[target_var],
                         )
        else:
            downs = analogs_seas_fcst(exp[target_var],
                                      srcobs[target_var],
                                      targetobs[target_var])

        print('exporting to .nc')
        downs.attrs = exp.attrs
        save_nc(downs,run_conf)
        save_nc(to_monthly(downs),run_conf)
    else:
        print(f'file {outdir+outfile} already exists, skipping computation')

def get_dir(system,tempfreq,arch_conf):
    
    """
    Parameters
    ----------
    system : str with the name of the system
             ex. era5, uerra, system5c3s.
    tempfreq : str with the time interval frequency of the predicitions 
             ex. daily_mean
    arch_conf : str with the path to the yml archive conf 
             ex. "run_conf.yml".
    """ 
    
    dirs = load_dirs(arch_conf)

    sys_dir = dirs['esarchive'] + dirs[system]['src'] + tempfreq + '/'
    sys_subdirs = dirs[system][tempfreq]
    print(f'sys_dir: {sys_dir}')
    
    return(sys_dir,sys_subdirs)

def get_domain(region,arch_conf):
    
    """
    Parameters
    ----------
    region : str with the name of the region to be selected
             ex. "vitigeoos-cat".
    arch_conf : str with the path to the yml archive conf 
             ex. "run_conf.yml".
    """ 
       
    with open(arch_conf) as f:    
        arch = yaml.load(f, Loader=yaml.FullLoader)
        domain = arch['domains'][region]
    
    return(domain)

def get_ldmonths(ld_timesteps,sd_month):
    
    ld_months = [ int(leadtime) + int(sd_month) for leadtime in ld_timesteps]
    ld_months = [ld-12 if ld > 12 else ld for ld in ld_months]
    
    return(ld_months)

#def preprocess(df):
#    df = fix_coords(df)
#    df = cut_domain(df,domain)
#    return(df)

def get_hcst(run_conf,arch_conf,region,fcst_sdate,ld_months,target_var):
    
    fx = load_fxconf(run_conf) # fixed conf

    # --------------------------
    #       Hcst files load
    # -------------------------

    exp_dir,exp_subdirs = get_dir(fx['exp_sys'],fx['temp_freq'],arch_conf)
    hcst_years = range(fx['hcst_years'][0],fx['hcst_years'][-1]+1)
    src_vars = fx['src_vars']
    sd_month = fcst_sdate[4:6]

    exp_files = [f'{exp_dir}/{exp_subdirs[var]}/{var}_{str(year)}{sd_month}01.nc'
                 for year in hcst_years
                 for var in src_vars[target_var]]

    exp = xr.open_mfdataset(exp_files,
                            preprocess=fix_coords,
                            drop_variables='time',
                            combine='nested',
                            concat_dim='sdate')

    # Creates sdate coord
    exp['sdate'] = pd.date_range('{}-{}-01'.format(hcst_years[0],sd_month), periods=len(hcst_years), freq='12MS')
    
    # New multidimensional coord leadtimes(sdate,time) with dates of each time step
    leadtimes = [pd.date_range('{}-{}-01'.format(year,sd_month), periods=len(exp.time), freq='D') for year in hcst_years]
    exp = exp.assign_coords(leadtimes=(('sdate', 'time'), leadtimes))
    exp['time'] = pd.date_range('{}-{}-01'.format('1900',sd_month), periods=len(exp.time), freq='D')


    # filters timesteps corresponding to lead season
    exp = exp.where(exp.leadtimes.dt.month.isin(ld_months), drop=True)
    #exp = exp.where(exp.leadtimes.dt.month == 2 and exp.leadtimes.dt.day == 29, drop=True)

    exp = exp.dropna(dim='time',how='any') # to drop 29th feb, look for another solution
    exp = exp.assign_attrs({'system':fx['exp_sys'],"target_var":target_var,
                            'tempfreq':fx['temp_freq'],'region':region,
                            'system-type':'hcst'})    
    return(exp)

def get_fcst(run_conf,arch_conf,region,fcst_sdate,ld_months,target_var):
    
    fx = load_fxconf(run_conf) # fixed conf

    # --------------------------
    #       Hcst files load
    # -------------------------

    exp_dir,exp_subdirs = get_dir(fx['exp_sys'],fx['temp_freq'],arch_conf)
    src_vars = fx['src_vars']
    sd_month = fcst_sdate[4:6]

    ## TODO: work with mutiple src_vars
    exp_files = f'{exp_dir}/{exp_subdirs[target_var]}/{target_var}_{fcst_sdate}.nc'

    exp = xr.open_dataset(exp_files,
                          drop_variables=['time','bnds','realization','time_bnds'])

    exp = fix_coords(exp)

    # Creates sdate coord
    #exp = exp.expand_dims({'sdate':datetime.strptime(fcst_sdate,"%Y%m%d")})
    #exp = exp.expand_dims('sdate')
    exp = exp.assign_coords(sdate=datetime.strptime(fcst_sdate,"%Y%m%d"))
    exp = exp.expand_dims('sdate')

    # New multidimensional coord leadtimes(sdate,time) with dates of each time step
    leadtimes = pd.date_range('{}-{}-01'.format(fcst_sdate[:4],fcst_sdate[4:6]), periods=len(exp.time), freq='D')
    exp = exp.assign_coords(leadtimes=(('time'), leadtimes))
    exp['time'] = leadtimes

    # filters timesteps corresponding to lead season
    exp = exp.where(exp.leadtimes.dt.month.isin(ld_months), drop=True)
    #exp = exp.where(exp.leadtimes.dt.month == 2 and exp.leadtimes.dt.day == 29, drop=True)

    exp = exp.dropna(dim='time',how='any') # to drop 29th feb, look for another solution
    exp = exp.assign_attrs({'system':fx['exp_sys'],"target_var":target_var,
                            'tempfreq':fx['temp_freq'],'region':region,
                            'system-type':'fcst'})   
    
    return(exp)



def get_obs(run_conf,arch_conf,target_var):
    
    fx = load_fxconf(run_conf) # fixed conf
    
    src_dir,src_subdirs = get_dir(fx['src_sys'],fx['temp_freq'],arch_conf)
    target_dir,target_subdirs = get_dir(fx['target_sys'],fx['temp_freq'],arch_conf)

    src_vars = fx['src_vars']
    training_years = range(fx['training_years'][0],fx['training_years'][-1]+1)

    #-----------------------------
    # Load predictors
    #-----------------------------
    flatten = lambda t: [item for sublist in t for item in sublist] 
    files = flatten([glob.glob(f'{src_dir}/{src_subdirs[var]}/{var}_{year}*.nc')
             for var in src_vars[target_var]
             for year in training_years])

    srcobs = xr.open_mfdataset(files,
                               preprocess = fix_coords,
                               drop_variables='time_bnds',
                               combine="by_coords",
                               join = 'left')
    #-----------------------------
    # Load target
    #-----------------------------

    files = flatten([glob.glob(f'{target_dir}/{target_subdirs[target_var]}/{target_var}_{year}*.nc')
             for year in training_years])

    targetobs = xr.open_mfdataset(files,
                                  drop_variables='time_bnds',
                                  preprocess = fix_coords,
                                  combine="by_coords",
                                  join = 'left')
  
    return(targetobs,srcobs)

def get_outfile(df,run_conf):
    arch = load_fxconf(run_conf)['outdir']
    
    outdir = '{}/output/{}/{}/{}/'.format(arch,df.attrs['system'],
                                          df.attrs['tempfreq'],
                                          df.attrs['target_var'])
    
    if df.attrs['system-type'] == "hcst":
    
        outfile = '{}-{}_{}-{}_{}_01.nc'.format(df.attrs['target_var'],
                                                df.attrs['region'],
                                                int(df.sdate.dt.year[0]),
                                                int(df.sdate.dt.year[-1]),
                                                int(df.sdate.dt.month[0]))
    elif df.attrs['system-type'] == "fcst":
        
        outfile = '{}-{}_{}.nc'.format(df.attrs['target_var'],
                                                df.attrs['region'],
                                                int(df.sdate.dt.strftime("%Y%m%d")))
    else:
        print("system-type not found")
                                                        
    return(outdir,outfile)

def save_nc(df,run_conf):
    
    outdir,outfile = get_outfile(df,run_conf)
    os.system(f'mkdir -p {outdir}')
    outdir,outfile
    
    df = df.transpose("time","ensemble","sdate","lat","lon")

    df.to_netcdf(outdir+outfile)
    
def to_monthly(df):
    
    attrs = df.attrs
    df = df.groupby('time.month').mean()
    df = df.rename({'month':'time'})
    df.attrs = attrs
    df.attrs['tempfreq'] = 'monthly_mean'
    
    return(df)